/*
 * pts.h:	P(resentation) T(ime) S(tamps) handling
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 *
 * Copyright (C) 2003 Werner Fink, <werner@suse.de>
 *
 * Changes and Additional Code:
 *
 * Copyright (C) 2003 Sven Goethel, <sven@jausoft.com>
 */

#ifndef _PTS_H
#define _PTS_H

#include <unistd.h>
#include <stdint.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include "types.h"

#define debug_pts(args...)
// #define debug_pts(args...)	fprintf(stderr, args)

enum ePTSstate { eOK = 0, eMISSED = -1, eDOSKIP = 1 };

class cPTS {
public:
    static const int MAX_REGS = 12;
    int            dtregs[MAX_REGS];
    struct timeval atregs[MAX_REGS];
    static const struct timeval atzero;
private:
    // to hold the current state
    uint_32 pts, stc;
    uint_32 l_pts, l_stc;
    uint_32 start_pts, start_stc;
    int    delay_sum;
    struct timeval ts;

public:
    static inline uint_32 PTSinMS(const uint_8 *pts)
    {
	return (uint_32)
	((   ((((uint64_t)pts[0]) & 0x0E) << 29)
	   | (( (uint64_t)pts[1])         << 22)
	   | ((((uint64_t)pts[2]) & 0xFE) << 14)
	   | (( (uint64_t)pts[3])         <<  7)
	   | ((((uint64_t)pts[4]) & 0xFE) >>  1))/90);
    }

/*

    sender (VDR)                    <->          receiver (spdif)
        |                                            |
     forward credits (ms)            ->           use credits if not expired
   + current time stamp for expire                if no data for credits repeat last frame

                                    <-            if expired put backward credits
                                                  of repeated frames

     if expired get backward credits
     for skipping                    ->           do skipping 
 */

    cPTS(void) 
    { clear(); }

    inline void mark(const uint_8 *ptsdata)
    { pts = PTSinMS(ptsdata); }

    void clear();

    inline bool    isARegTimeZero(int a)
    {
        if ( 0<=a&&a<MAX_REGS && 
             ( atregs[a].tv_sec!=0 || atregs[a].tv_usec!=0 ) )
		return false;
	return true;
    }
    inline int getARegTimeDiff(int a, int b)
    {
    	int td=0;
        if ( 0<=a&&a<MAX_REGS && 0<=b&&b<MAX_REGS ) {
		struct timeval diff;
		timersub(&atregs[a], &atregs[b], &diff);
		td = (diff.tv_sec*1000+diff.tv_usec/1000);
		if(td!=0)
	           debug_pts("PTS getARegTimeDiff %d-%d=%dms\n", a,b,td);
	}
	return td;
    }
    inline int getLastTimeDiff()
    {
    	int td;
	struct timeval cur, diff;
	gettimeofday(&cur, 0);
	timersub(&cur, &ts, &diff);
	td = (diff.tv_sec*1000+diff.tv_usec/1000);
	gettimeofday(&ts, 0);
	return td;
    }
    inline int getDelaySum()
    {
	return delay_sum;
    }
    inline int setDelaySum(int d)
    {
	delay_sum=d;
	debug_pts("PTS setDelaySum to %d\n", d);
	return delay_sum;
    }
    inline int addDelaySum(int d)
    {
        if(d!=0)
	    debug_pts("PTS addDelaySum %d+%d=%d\n", delay_sum, d, delay_sum+d);
	delay_sum+=d;
	return delay_sum;
    }
    inline uint_32 delay()
    {
	return stc - pts;
    };
    inline uint_32 getSTC() { return stc; }

    inline uint_32 getPTS() { return pts; }

    
    int process( const uint_8 *ptsdata, uint_32 stc, int sleep_threshold);

    int process( uint_32 stc, int sleep_threshold );
};

#endif

